﻿// MainForm.cs
//
//  Copyright (C) 2009 Mawerick
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml;

using Twp.Utilities;

namespace GUISelect
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		internal void OnLoad( object sender, EventArgs e )
		{
			Program.Settings.Load();

			string path = Program.Settings.AOPath;
			if( AO.ConfirmPath( ref path ) )
				Program.Settings.AOPath = path;
			else
				Application.Exit();

			this.UpdatePath( Program.Settings.AOPath );
			this.SetSkinBox.Checked = Program.Settings.SetSkin;
		}

		internal void OnClosing( object sender, FormClosingEventArgs e )
		{
			Program.Settings.Save();
		}

		internal void OnAbout( object sender, EventArgs e )
		{
			AboutDialog about = new AboutDialog();
			about.Url = "http://guiselect.wrongplace.net/";
			about.Email = "mawerick@wrongplace.net";
			about.Image = Resources.Resource.Icon_64;
			about.LicenseImage = Resources.Resource.Icon_GPL;
			about.VersionFormat = VersionFormat.Simple;
			about.ShowDialog();
			about.Dispose();
		}
		
		internal void OnBrowsePath( object sender, EventArgs e )
		{
			string path = Program.Settings.AOPath;
			if( AO.BrowseAOPath( ref path ) )
				Program.Settings.AOPath = path;
		}

		internal void OnSetSkinChanged( object sender, EventArgs e )
		{
			Program.Settings.SetSkin = this.SetSkinBox.Checked;
		}

		internal void OnSave( object sender, EventArgs e )
		{
			this.SaveGui();
			Application.Exit();
		}

		internal void UpdatePath( string path )
		{
			if( !AO.IsPathValid( Program.Settings.AOPath ) )
			{
				Log.Warning( "The selected folder is not a valid Anarchy Online installation." );
				this.AOPathEntry.Text = "**Error**";
				return;
			}

			Program.Settings.AOPath = path;
			this.AOPathEntry.Text = path;

			this.GuiList.Items.Clear();
			string[] guiDirs = AO.GetGuiDirs( Program.Settings.AOPath );
			if( guiDirs != null )
			{
				this.GuiList.Items.AddRange( guiDirs );
				if( guiDirs.Length > 1 )
				{
					this.GuiLabel.Enabled = true;
					this.GuiList.Enabled = true;
					this.SetSkinBox.Enabled = true;
					this.SaveButton.Enabled = true;
				}
				else
				{
					this.GuiLabel.Enabled = false;
					this.GuiList.Enabled = false;
					this.SetSkinBox.Enabled = false;
					this.SaveButton.Enabled = false;
				}
				this.LoadGui();
			}
			else
			{
				Log.Warning( "No GUI's found in the selected AO folder!" );
				this.GuiLabel.Enabled = false;
				this.GuiList.Enabled = false;
				this.SetSkinBox.Enabled = false;
				this.SaveButton.Enabled = false;
			}
		}

		internal void UpdateGui( string gui )
		{
			Log.Debug( "[Main] [UpdateGui]", "GUI: " + gui );
			if( this.GuiList.Items.Contains( gui ) )
				this.GuiList.SelectedItem = gui;
			else
				this.GuiList.SelectedItem = "Default";
		}

		internal void LoadGui()
		{
			string path = System.IO.Path.Combine( Program.Settings.AOPath, "Prefs" );
			if( !Directory.Exists( path ) )
				return;

			string fileName = System.IO.Path.Combine( path, "Prefs.xml" );
			if( !File.Exists( fileName ) )
				return;

			XmlDocument doc = new XmlDocument();
			try
			{
				doc.Load( fileName );
			}
			catch( FileNotFoundException ex )
			{
				Log.Debug( "[Main] [LoadGui]", ex.Message );
				this.UpdateGui( "Default" );
				return;
			}

			XmlNodeList nodes = doc.GetElementsByTagName( "Value" );
			for( int i = 0; i < nodes.Count; ++i )
			{
				XmlAttributeCollection ac = nodes[i].Attributes;
				XmlAttribute name = ac["name"];
				if( name.Value == "GUIName" )
				{
					XmlAttribute value = ac["value"];
					this.UpdateGui( value.Value.Trim( new char[] { '"' } ) );
					break;
				}
			}
		}

		internal void SaveGui()
		{
			string path = System.IO.Path.Combine( Program.Settings.AOPath, "Prefs" );
			if( !Directory.Exists( path ) )
				Directory.CreateDirectory( path );

			string fileName = System.IO.Path.Combine( path, "Prefs.xml" );
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.Load( fileName );
			}
			catch( FileNotFoundException )
			{
				Log.Information( "Failed to load Prefs.xml file. Attempting to create one from the default MainPrefs.xml file..." );
				try
				{
					doc.Load( AO.GetGuiFile( Program.Settings.AOPath, "Default", "MainPrefs.xml" ) );
				}
				catch( FileNotFoundException )
				{
					Log.Error( "Failed to load MainPrefs.xml file. Unable to save GUI selection. Your Anarchy Online installation may be corrupt!" );
					return;
				}
			}

			XmlNodeList nodes = doc.GetElementsByTagName( "Value" );
			for( int i = 0; i < nodes.Count; ++i )
			{
				XmlAttributeCollection ac = nodes[i].Attributes;
				XmlAttribute name = ac["name"];
				if( name.Value == "GUIName" )
				{
					XmlAttribute value = ac["value"];
					value.Value = "\"" + this.GuiList.SelectedItem.ToString() + "\"";
				}
				if( name.Value == "GUISkin" && Program.Settings.SetSkin == true )
				{
					string skin = ReadGuiSkin();
					if( !String.IsNullOrEmpty( skin ) )
					{
						XmlAttribute value = ac["value"];
						value.Value = "\"" + skin + "\"";
					}
				}
			}
			doc.Save( fileName );
		}

		internal string ReadGuiSkin()
		{
			string gui = this.GuiList.SelectedItem.ToString();
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.Load( AO.GetGuiFile( Program.Settings.AOPath, gui, "MainPrefs.xml" ) );
			}
			catch( FileNotFoundException )
			{
				Log.Error( "Failed to load \"MainPrefs.xml\" in the selected GUI ({0}).", gui );
				return null;
			}

			string skin = null;
			XmlNodeList nodes = doc.GetElementsByTagName( "Value" );
			for( int i = 0; i < nodes.Count; ++i )
			{
				XmlAttributeCollection ac = nodes[i].Attributes;
				XmlAttribute name = ac["name"];
				if( name.Value == "GUISkin" )
				{
					XmlAttribute value = ac["value"];
					skin = value.Value.Trim( new char[] { '"' } );
					break;
				}
			}
			Log.Debug( "[Main] [ReadGuiSkin]", "Skin: " + skin );
			if( !String.IsNullOrEmpty( skin ) )
			{
				FileInfo skinInfo = new FileInfo( AO.GetCDImageDir( Program.Settings.AOPath, skin ) );
				skin = skinInfo.FullName;
			}
			if( !File.Exists( skin ) )
				skin = null;
			return skin;
		}
	}
}
